import { TransactionResult } from '../../utils/transactions';
import TransactionManager, { ExecTransactionProps, TransactionManagerConfig } from '../TransactionManager';
interface SafeRelayTransactionManagerConfig {
    url: string;
}
declare class SafeRelayTransactionManager implements TransactionManager {
    url: string;
    /**
     * Initializes an instance of the Safe Relay Transaction Manager.
     *
     * @param options - The URL pointing to the Safe Transaction Service
     * @returns The SafeRelayTransactionManager instance
     */
    constructor(options: SafeRelayTransactionManagerConfig);
    /**
     * Returns the configuration of the Safe Relay Transaction Manager.
     *
     * @returns The name of the TransactionManager in use and the URL of the service
     */
    get config(): TransactionManagerConfig;
    /**
     * Executes a list of transactions via the Safe Transaction Relay.
     *
     * @param options
     * @returns The transaction response
     */
    execTransactions({ ownerAccount, safeExecTxParams, contractManager, ethLibAdapter, isConnectedToSafe }: ExecTransactionProps): Promise<TransactionResult>;
    private getTransactionEstimations;
    private sendTransactionToRelay;
    private signTransactionHash;
}
export default SafeRelayTransactionManager;
